# PIPA

PIPA (Progressive & Intelligent Performance Analytics) is a platform that aggregates a complete toolchain of performance data collection, processing, and analysis with
 advanced algorithms, enabling users to effortlessly obtain in - depth insights into the performance of their systems and applications. It bridges the gap between raw performance data and actionable information, allowing for quick identification of bottlenecks and optimization opportunities.


<div align="center">
    <img src="asset/logo.png" width="400" height="400">
</div>

PIPA (枇杷, loquat) is a local fruit of Zhejiang, China.
PIPA consists of three parts: loquat tree, flower and fruit, which represent the collecting & processing, analysis and conclusion of performance data respectively.

PIPA is still in the active development process, and the current development focus is on the loquat tree.

![GitHub License](https://img.shields.io/github/license/ZJU-SPAIL/pipa)
![GitHub Actions Workflow Status](https://img.shields.io/github/actions/workflow/status/ZJU-SPAIL/pipa/main.yml)
![GitHub top language](https://img.shields.io/github/languages/top/ZJU-SPAIL/pipa)
[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)
[![Coverage Status](https://coveralls.io/repos/github/ZJU-SPAIL/pipa/badge.svg?branch=main)](https://coveralls.io/github/ZJU-SPAIL/pipa?branch=main)


## Features

- **Data Collecting**: PIPA can collect data from a variety of sources, using tools like perf, sar, and more. It supports multiple platforms including x86_64, ARM, and RISC-V, making it versatile and adaptable. Currently PIPA is capable of collecting and parsing perf and sar data, providing detailed performance metrics.
- **Script Generation**: To reduce the noise generated by the Python runtime, PIPA can generate scripts that collect performance data. 
- **Data Processing**: PIPA can process the collected performance data, including alignment and segmentation, to serve meaningful analysis.
- **Data Visualization**: PIPA can visualize based on the performance data collected to provide intuitive insights.
- **Data Analytics**: PIPA will integrate SPAIL's performance methodology and models to provide meaningful analysis and reveal software and hardware bottlenecks.

## Preparation

```pipa``` needs to run on ```python 3```, and you may need to install ```venv``` (virtual environemnt) package to isolate ```pipa``` from other python projects, prior to that, you also need to have ```pip``` installed on your computer.

### Install PIP
Here is a common way to install ```pip``` on a computer without any package management tool:
#### 1. Download get-pip.py:
Download the [get-pip.py](https://bootstrap.pypa.io/get-pip.py) script.
#### 2. Open a Terminal/Command Prompt:
Open a terminal on Linux or macOS, or a command prompt on Windows.
#### 3. Navigate to the Download Directory:
Use the ```cd``` command to navigate to the directory where you saved get-pip.py.
#### 4. Run the Script:
Execute the script using the appropriate Python command:
- Linux/macOS:
  ```sh
  python3 get-pip.py
  ```
- Windows:
  > If Python 3 is the default
  ```sh
  python get-pip.py
  ```
  > If Python 3 is explicitly specified
  ```sh
  python3 get-pip.py
  ```
### Install VENV

Then you can install ```venv``` with a single command line:
```sh
pip install venv
```
### Setup and Use VENV

Please find [more instructions](https://packaging.python.org/en/latest/guides/installing-using-pip-and-virtual-environments/) about how to setup and use virtual environment from the python official website if you are new to it.

## Installation

PIPA can be easily installed using pip:

```sh
pip install PyPIPA
```


## Quickstart

After installation, you can start using PIPA to collect, integrate, and analyze your data. 

To generate a script that collect performance data, you only need to use:

```sh
pipa generate
```
Then you can complete the interaction through the CLI to provide the necessary parameters. You can choose to start the workload with perf, or you can choose to observe the system directly.

For the detailed usage, please check [user guide](doc/User-guide.md).

## Build

To build PIPA, you can use the `python` command with the `build` module:  `python -m build`, we use `hatchling` as the build backend.

## LICENSE

PIPA is distributed under the terms of the [MIT License](LICENSE).


## Contributing

Contributions to PIPA are always welcome. Whether it's feature enhancements, bug fixes, or documentation, your contributions are greatly appreciated.
